//---------------------------------------------------
// module  : f_BIEDA.cc                       
// auteur  : Max Mignotte                            
// date    :                                              
// langage : C++                                      
// labo    :                                  
//---------------------------------------------------

//-----------------------------------------------
// FICHIERS INCLUS ------------------------------
//-----------------------------------------------
#include "BIEDA.h"

//------------------------------------------------
// METHODES DE LA CLASSE KMEANCONSTRAIN  ---------
//------------------------------------------------
//----------------------------------------------------------
// Constructeur de la classe BIEDA
//----------------------------------------------------------
BIEDA::BIEDA(int*** img,int lgth,int wdth)           
 {
  int i,j,k;

  //Initialisation
  length=lgth;
  width=wdth;

  //Allocation Memoire Matrice 
  mat_img=imatrix_allocate_3d(TROIS,length,width);
  mat_seg=fmatrix_allocate_2d(length,width);
  mat_cont1=fmatrix_allocate_2d(length,width);
  mat_cont2=fmatrix_allocate_2d(length,width);
  mat_cont3=fmatrix_allocate_2d(length,width);
  mat_cont4=fmatrix_allocate_2d(length,width);
  mat_cont5=fmatrix_allocate_2d(length,width);
  mat_cont6=fmatrix_allocate_2d(length,width);

  //Initialisation
  for(k=0;k<TROIS;k++) for(i=0;i<length;i++) for(j=0;j<width;j++) 
  mat_img[k][i][j]=img[k][i][j];  
 }


//----------------------------------------------------------
// Destructeur de la classe BIEDA
//----------------------------------------------------------
BIEDA::~BIEDA()
{ 
  //Libere Memoire Matrice 
  if (mat_img)    free_imatrix_3d(mat_img,TROIS);
  if (mat_seg)    free_fmatrix_2d(mat_seg);

  if (mat_cont1)  free_fmatrix_2d(mat_cont1);
  if (mat_cont2)  free_fmatrix_2d(mat_cont2);
  if (mat_cont3)  free_fmatrix_2d(mat_cont3);
  if (mat_cont4)  free_fmatrix_2d(mat_cont4);
  if (mat_cont5)  free_fmatrix_2d(mat_cont5);
  if (mat_cont6)  free_fmatrix_2d(mat_cont6);
}

//------------------------------------------------
// METHODES DE LA CLASSE BIEDA -------------------
//------------------------------------------------
//----------------------------------------------------------
// DetectContour
//----------------------------------------------------------
void BIEDA::DetectContour()
{
 int i,j,l;
 char MyBuf[100];
 
 //>Major Parameter
 const int NB_LINES=180; 
 
 const int NB_RESOL=4; 
 const int NB_DEFOR=5*2; 
 const int NB_BCOUL=6;  
 const float RES_MIN=0.6; 
 const float RES_MAX=0.7;
 
 //Minor Parameter
 const float PROP_WND=0.80;
 const float THR_EPS=-20.0; 

 const float ADDIMGCART=0.325;
 const int   NBGAUSSFILTPOT=1;
 const int   REMOVEISOLPIX=1;
 const int   REMOVELASTLINE=5;

 //---Other Paramter---
 //--AddGradImg--Cst [1.0]
 //--GaussianFilter-Img-Cst [3]
 //--Epsilon--Cst [0.75]
 
 //>Presentation
 printf("\n ========");
 printf("\n BIEDA");
 printf("\n ========");
 printf("\n [lgth=%d | wdth=%d]",length,width);

 //>MemoryAllocation  
 int***  ImgTransf1=imatrix_allocate_3d(TROIS,length,width);
 int***  ImgTransf2=imatrix_allocate_3d(TROIS,length,width);
 int***  ImgTransf3=imatrix_allocate_3d(TROIS,length,width);
 float*** ImgCont1=fmatrix_allocate_3d(NB_DEFOR,length,width);
 float*** ImgCont2=fmatrix_allocate_3d(NB_DEFOR,length,width);
 int**    TabPts=imatrix_allocate_2d(NB_DEFOR,2);      
 int**    TabPtsInc=imatrix_allocate_2d(NB_DEFOR,2);   
 float**  ImgF=fmatrix_allocate_2d(length,width);
 float**  ImgT=fmatrix_allocate_2d(length,width);
 float**  ImgFilt=fmatrix_allocate_2d(length,width);
 float**  ImgProfile=fmatrix_allocate_2d(length,width);

 //>Init
 for(l=0;l<TROIS;l++) for(i=0;i<length;i++) for(j=0;j<width;j++)
    { ImgTransf1[l][i][j]=0;
      ImgTransf2[l][i][j]=0; 
      ImgTransf3[l][i][j]=0; }

 for(l=0;l<NB_DEFOR;l++) for(i=0;i<length;i++) for(j=0;j<width;j++)
    { ImgCont1[l][i][j]=0.0;
      ImgCont2[l][i][j]=0.0; }

 for(i=0;i<length;i++) for(j=0;j<width;j++) 
    { ImgF[i][j]=0.0;
      ImgT[i][j]=0.0; 
      ImgFilt[i][j]=0.0; }

 
 //================================================
 //[0]>Pre-Traitment
 //================================================
 RestorLastLine(mat_img,mat_img,length,width);
 fflush(stdout);

 //================================================
 //[1]>PolarDeformation/LSD
 //================================================
 //>SamplingPos   
 FindRegSpacedPts_2(length,width,PROP_WND,NB_DEFOR/2,TabPts,TabPtsInc); 
 for(l=0;l<NB_DEFOR/2;l++) printf(" [%d::%d]",TabPts[l][0],TabPts[l][1]);

 //>PolarDeformations
 for(l=0;l<NB_DEFOR/2;l++) 
    { printf("\n\n <<[%d]>>",l);   
      PolarMapping(mat_img,ImgTransf1,length,width,TabPtsInc[l][0],TabPtsInc[l][1],0);  
      FillHoleImg(ImgTransf1,length,width,1);
      ComputeEdge(ImgTransf1,length,width,ImgCont1[l],NB_BCOUL,RES_MIN,RES_MAX,NB_RESOL,NB_LINES,THR_EPS);
      Transf2DMat3DMat(ImgCont1[l],ImgTransf2,length,width);
      PolarMapping(ImgTransf2,ImgTransf3,length,width,TabPtsInc[l][0],TabPtsInc[l][1],1);   
      Transf3DMat2DMat(ImgTransf3,ImgCont2[l],length,width); }

 //>LogPolarDeformations
 for(l=NB_DEFOR/2;l<NB_DEFOR;l++) 
    { printf("\n\n >>[%d]<<",l);   
      LogPolarMapping(mat_img,ImgTransf1,length,width,TabPtsInc[l-(NB_DEFOR/2)][0],TabPtsInc[l-(NB_DEFOR/2)][1],0);  
      FillHoleImg(ImgTransf1,length,width,1);
      ComputeEdge(ImgTransf1,length,width,ImgCont1[l],NB_BCOUL,RES_MIN,RES_MAX,NB_RESOL,NB_LINES,THR_EPS);
      Transf2DMat3DMat(ImgCont1[l],ImgTransf2,length,width);
      LogPolarMapping(ImgTransf2,ImgTransf3,length,width,TabPtsInc[l-(NB_DEFOR/2)][0],TabPtsInc[l-(NB_DEFOR/2)][1],1);   
      Transf3DMat2DMat(ImgTransf3,ImgCont2[l],length,width); }

 //===============================================
 //[2]>Averaging+ImgWithoutDeformation
 //===============================================
 AverageMat(ImgCont2,ImgF,length,width,NB_DEFOR); 
 Recal(ImgF,length,width);

 ComputeEdge(mat_img,length,width,ImgProfile,NB_BCOUL,RES_MIN,RES_MAX,NB_RESOL,NB_LINES,THR_EPS);
 Recal(ImgProfile,length,width);
 AddMatrix(ImgF,ImgProfile,ImgT,length,width,ADDIMGCART);
 Recal(ImgT,length,width);
 copy_mat(ImgT,ImgF,length,width); 

 //===============================================
 //[3]>NonMaxLocSup/Cleaning
 //===============================================
 copy_mat(ImgF,ImgFilt,length,width);
 Recal(ImgFilt,length,width);
 GaussianFilter2N(ImgFilt,length,width,NBGAUSSFILTPOT);
 NonMaxLocSuppression(ImgFilt,mat_img,mat_seg,length,width,0);
 RemoveLastLineColumn(mat_seg,length,width,REMOVELASTLINE);        
 Recal(mat_seg,length,width);
 RemoveIsolatedPix(mat_seg,length,width,REMOVEISOLPIX);

 //=End===================================================================================
 //>Save
 i=strlen(Name_Img);
 strncpy(MyBuf,Name_Img,i-4);
 SaveImagePgm(MyBuf,"",mat_seg,length,width); 

 //>FreeMemory
 if (ImgTransf1) free_imatrix_3d(ImgTransf1,TROIS);
 if (ImgTransf2) free_imatrix_3d(ImgTransf2,TROIS);
 if (ImgTransf3) free_imatrix_3d(ImgTransf3,TROIS);
 if (ImgCont1) free_fmatrix_3d(ImgCont1,NB_DEFOR);
 if (ImgCont2) free_fmatrix_3d(ImgCont2,NB_DEFOR);
 if (TabPts) free_imatrix_2d(TabPts);
 if (TabPtsInc) free_imatrix_2d(TabPtsInc);
 if (ImgF) free_fmatrix_2d(ImgF);
 if (ImgT) free_fmatrix_2d(ImgT);
 if (ImgFilt) free_fmatrix_2d(ImgFilt);
 if (ImgProfile) free_fmatrix_2d(ImgProfile);
}

//----------------------------------------------------------
// ComputeEdge [Mat_SoftCont calcul] In:ImgIn  Out:ImgOut
// -------------------------------------------------------
//----------------------------------------------------------
void BIEDA::ComputeEdge(int*** ImgIn,int lgth,int wdth,float** ImgOut,int NbCoul,float ResMin,float ResMax,\
int NbRes,int NbLinMax,float eps)
{
 int i,j;
 
 //>AllocationMemoryUsed
 float**   Mat_Cnt1=fmatrix_allocate_2d(lgth,wdth);
 float**   Mat_SoftCont1=fmatrix_allocate_2d(lgth,wdth);
 float**   Mat_SoftCont2=fmatrix_allocate_2d(lgth,wdth);

 //>Init
 for(i=0;i<lgth;i++) for(j=0;j<wdth;j++)  
    { Mat_Cnt1[i][j]=0.0;
      Mat_SoftCont1[i][j]=0.0;
      Mat_SoftCont2[i][j]=0.0; }

 //EDGE ==========
 //>[Pot+Grad]+Lsd_(dist-div-SzW)
 GradTexMultiColor__(ImgIn,Mat_SoftCont2,lgth,wdth,3,5,7,NbCoul);    
 AddGradImg(ImgIn,Mat_SoftCont2,Mat_SoftCont1,lgth,wdth,1.0);
 My_LineSegDetector_2(ImgIn,lgth,wdth,Mat_Cnt1,Mat_SoftCont1,eps,ResMin,ResMax,NbCoul,NbRes,NbLinMax);
 Recal(Mat_Cnt1,lgth,wdth);
 copy_mat(Mat_Cnt1,ImgOut,length,width);
 
 //>Libere Memoire Matrice  
 if (Mat_SoftCont1)   free_fmatrix_2d(Mat_SoftCont1);
 if (Mat_SoftCont2)   free_fmatrix_2d(Mat_SoftCont2);
 if (Mat_Cnt1)        free_fmatrix_2d(Mat_Cnt1);
}




 
